#!/usr/bin/env python
# coding=utf-8

# This script generates resistor-arrays of various sizes. Documentation on the
# usage will follow. See: http://forum.fritzing.org/t/2079

import svgwrite as svg
from svgwrite.base import BaseElement
from svgwrite.mixins import Presentation, Markers, Transform
import os


STAR = 1
PARALLEL = 3


# Generates a breadboard-view of the element and saves it to a file.
# @param output File to save the view to
# @param pins   Number of pins to generate
# @param label  Label to write on top
def gen_breadboard(output, resistors, circuittype, resistance):
    dx = 3229             # Difference between each pin
    min_width = 8902      # Base width of the viewbox for a three-pin-object
    width_multiplicator = 0.275661 / min_width
                          # Used to calculate the width of the SVG in inches
    pins = pin_number(resistors, circuittype)

    # Calculate the viewbox
    width = 8902 + (pins - 3) * dx
    temp_name = "temp.svg"
    draw = svg.Drawing(temp_name,
        size = (str(width * width_multiplicator) + "in", "0.334976in"),
        viewBox = "0 0 " + str(width) + " 10818")

    # Center expandable body
    expand = (pins - 3) * dx
    path = [
        ("M", 5951 + expand, 5072),
        ("l", -15, 2, -2956 - expand, 0, 0, -5062),
        ("c", 948, -14, 2030, -15, 2971 + expand, -2),
        ("l", 0, 5063),
        ("z")
    ]
    draw.add(draw.path(d = path, fill = "#EB9922"))
    path = [
        ("M", 2980, 12),
        ("c", 947, -14, 2028, -16, 2971 + expand, -3),
        ("l", 0, 336),
        ("c", -943, -13, -2024, -11, -2971 - expand, 3),
        ("l", 0, -336),
        ("z")
    ]
    draw.add(draw.path(d = path, fill = "#D3891E"))

    # Right end
    path = [
        ("M", 8780 + expand, 5037),
        "c0,379 -227,620 -621,620l-998 0c-566,-883 1409,0 0,0 -230,-358 -631,-596 -1088,-596 -55,0 -109,3 -162,10l0 -5063c1104,16 2010,51 2248,108 333,81 507,313 621,621 275,746 0,4299 0,4299z"
    ]
    draw.add(draw.path(d = path, fill = "#EB9922"))
    path = [
        ("M", 5912 + expand, 9),
        "c1112,15 2024,51 2263,109 333,81 507,313 621,621 100,270 120,940 97,1627 -5,-567 -14,-1064 -97,-1290 -114,-308 -287,-540 -621,-621 -239,-57 -1151,-93 -2263,-109l0 -336zm2868 5028c0,379 -227,620 -621,620 -1752,0 -651,0 -998,0 -230,-358 -631,-596 -1088,-596 -55,0 -109,3 -162,10l0 -798c55,-8 109,-12 162,-12 462,0 1029,325 1343,723 137,180 -782,-981 -46,-17 262,336 640,132 754,72 321,-205 457,-449 506,-651 130,-528 246,-1374 263,-2787 24,1311 -114,3436 -114,3436z"
    ]
    draw.add(draw.path(d = path, fill = "#D3891E"))
    path = [
        ("M", 8403 + expand, 627),
        "c-252,-188 -1002,-48 -1006,237 776,466 703,2617 696,2887 1,582 346,642 375,643 257,5 573,-3290 -65,-3767z"
    ]
    draw.add(draw.path(d = path, fill = "#FEFEFE", fill_opacity="0.549020", enable_background="new"))

    # Generate the bottom pins
    params = { "fill": "none", "stroke": "#96989A", "stroke_width": "645.887", "stroke_linecap": "round" }
    for i in range(pins):
        x = 1229 + dx * i

        # Generate the repeated middle content
        if (i >= 2):
            shift = dx * (i - 2)
            path = [
                ("M", 6073 + shift, 5062),
                "c-457,0 -859,237 -1088,596l-1053 0c-230,-358 -631,-596 -1088,-596l0 0 0 -1265 3230 0 0 1265z"
            ]
            draw.add(draw.path(d = path, fill = "#EB9922"))
            path = [
                ("M", 6073 + shift, 4262),
                "l0 800c-457,0 -859,237 -1088,596l-1053 0c-230,-358 -631,-596 -1088,-596l0 0 0 -800 0 0c459,0 1030,324 1343,723 180,240 362,231 544,0 312,-397 881,-723 1343,-723z"
            ]
            draw.add(draw.path(d = path, fill = "#D3891E"))


        # Place the actual legs
        draw.add(draw.line((x, 6332), (x, 10495), **params))
        draw.add(draw.rect((x - 720, 5657), (1439, 646), fill = "#727376"))
        draw.add(draw.rect((x - 513, 5657), (194, 646), fill = "#E6E7E8"))

        # Place the connector (which will get it's gorn-value later...)
        draw.add(draw.rect((x - 323, 9526), (646, 1292), id = "connector" + str(i) + "pin", fill = "none", class_="0.0." + str(i)))


    # Left end
    draw.add(draw.path(fill = "#EB9922", d = "M3020 5074c-58,-8 -116,-12 -176,-12 -457,0 -859,237 -1088,596l-1029 0c-401,0 -621,-598 -621,-621 0,0 -237,-3553 0,-4298 104,-327 318,-563 621,-621 278,-53 1194,-89 2294,-106l0 5062z"))
    draw.add(draw.path(fill = "#D3891E", d = "M3020 5074c-57,-8 -116,-12 -176,-12 -457,0 -859,237 -1088,596l-1029 0c-401,0 -621,-598 -621,-621 0,0 -124,-1862 -103,-3173 37,1186 99,2307 148,2483 93,344 257,492 558,709 239,107 518,228 792,-70 313,-398 884,-723 1343,-723 57,0 116,5 176,15l0 797zm-3004 -2716c-5,-707 18,-1348 105,-1619 104,-327 318,-563 621,-621 277,-53 1185,-89 2278,-106l0 336c-1093,17 -2001,53 -2278,106 -303,58 -517,294 -621,621 -72,227 -100,715 -105,1283z"))
    draw.add(draw.path(fill = "#FEFEFE", fill_opacity="0.549020", enable_background = "new", d="M588 550c396,-177 1243,-28 1226,283 -976,0 -653,2238 -933,2876 -197,448 -275,570 -579,680 -151,-43 -394,-3535 286,-3839z"))

    # Add the text on top
    label = gen_label(resistors, circuittype, resistance)
    draw.add(draw.text(label, x = [ 2582 ], y = [ 3339 ], fill = "#5F6160", font_weight = "normal", font_size = "2242.68", font_family = "Droid Sans"))

    # Save the file
    draw.save()

    # Load the file to add the gorn-values (not possible with svgwrite) to the connectors
    with open(output, "wt") as fout:
        with open(temp_name, "rt") as fin:
            for line in fin:
                fout.write(line.replace('class="0.0.', 'gorn="0.0.'))
    os.remove(temp_name)

def gen_label(resistors, circuittype, resistance):
    res = str(resistance)
    res_code = res[:2] + str(len(res) - 2)
    n = pin_number(resistors, circuittype)
    return "L {:02d} {} {}".format(n, circuittype, res_code)

def pin_number(resistors, circuittype):
    return resistors + 1 if circuittype == STAR else resistors * 2

if not os.path.exists("build"): os.makedirs("build")
for i in range(3, 6):
    gen_breadboard("build/parallel_{}.svg".format(i), i, PARALLEL, 470)
for i in range(4, 10):
    gen_breadboard("build/star_{}.svg".format(i), i, STAR, 10000)
